﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.BusinessLogicsContracts;
using ConfectioneryContracts.SearchModels;
using ConfectioneryContracts.StoragesContracts;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDataModels.Enums;
using ConfectioneryBusinessLogic.MailWorker;
using Microsoft.Extensions.Logging;
using System.Xml.Linq;

namespace ConfectioneryBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;

        private readonly IOrderStorage _orderStorage;

        private readonly AbstractMailWorker _mailWorker;

        static readonly object locker = new();

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker mailWorker)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _mailWorker = mailWorker;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }
            model.Status = OrderStatus.Принят;
            var element = _orderStorage.Insert(model);
            if (element == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            Task.Run(() => _mailWorker.MailSendAsync(new MailSendInfoBindingModel
            {
                MailAddress = element.ClientEmail,
                Subject = $"Новый заказ создан. Номер заказа - {element.Id}",
                Text = $"Заказ №{element.Id} от {element.DateCreate} на сумму {element.Sum} принят."
            }));
            return true;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderId: {Id}.", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id: {Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            lock (locker)
            {
                return ChangeStatus(model, OrderStatus.Выполняется);
            }
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.PastryId <= 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор выпечки", nameof(model.PastryId));
            }
            if (model.ClientId <= 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор клиента", nameof(model.ClientId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("В заказе должно быть хотя бы одна выпечка", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Стоимость заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. Id: {Id}. Sum: {Sum}. PastryId: {PastryId}. PastryCount: {Count}", model.Id, model.Sum,
                model.PastryId, model.Count);
        }

        private bool ChangeStatus(OrderBindingModel model, OrderStatus newStatus)
        {
            CheckModel(model, false);
            var order = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (order == null)
            {
                _logger.LogWarning("Change status operation failed. Order not found");
                return false;
            }
            if (order.Status + 1 != newStatus)
            {
                _logger.LogWarning("Change status operation failed. Incorrect new status: {newStatus}. Current status: {currStatus}",
                    newStatus, order.Status);
                return false;
            }
            model.Status = newStatus;
            if (order.ImplementerId.HasValue)
            {
                model.ImplementerId = order.ImplementerId;
            }
            if (model.Status == OrderStatus.Готов)
            {
                model.DateImplement = DateTime.Now;
            }
            else
            {
                model.DateImplement = order.DateImplement;
            }
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }
            string DateInfo = model.DateImplement.HasValue ? $"Дата выполнения {model.DateImplement}" : "";
            Task.Run(() => _mailWorker.MailSendAsync(new MailSendInfoBindingModel
            {
                MailAddress = order.ClientEmail,
                Subject = $"Заказ №{order.Id}",
                Text = $"Заказ №{order.Id} изменен статус на {model.Status}. {DateInfo}"
            }));
            return true;
        }
    }
}
